(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports);
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports);
    global.includeFragmentElement = mod.exports;
  }
})(this, function (exports) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  var _createClass = function () {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }

    return function (Constructor, protoProps, staticProps) {
      if (protoProps) defineProperties(Constructor.prototype, protoProps);
      if (staticProps) defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();

  function _possibleConstructorReturn(self, call) {
    if (!self) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }

    return call && (typeof call === "object" || typeof call === "function") ? call : self;
  }

  function _inherits(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
      throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
    }

    subClass.prototype = Object.create(superClass && superClass.prototype, {
      constructor: {
        value: subClass,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
    if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
  }

  function _CustomElement() {
    return Reflect.construct(HTMLElement, [], this.__proto__.constructor);
  }

  ;
  Object.setPrototypeOf(_CustomElement.prototype, HTMLElement.prototype);
  Object.setPrototypeOf(_CustomElement, HTMLElement);
  /* eslint-disable github/no-then */

  var privateData = new WeakMap();

  function fire(name, target) {
    setTimeout(function () {
      var event = target.ownerDocument.createEvent('Event');
      event.initEvent(name, false, false);
      target.dispatchEvent(event);
    }, 0);
  }

  function handleData(el) {
    return getData(el).then(function (html) {
      var parentNode = el.parentNode;
      if (parentNode) {
        el.insertAdjacentHTML('afterend', html);
        parentNode.removeChild(el);
      }
    }, function () {
      el.classList.add('is-error');
    });
  }

  function getData(el) {
    var src = el.src;
    var data = privateData.get(el);
    if (data && data.src === src) {
      return data.data;
    } else {
      if (src) {
        data = el.load();
      } else {
        data = Promise.reject(new Error('missing src'));
      }
      privateData.set(el, { src: src, data: data });
      return data;
    }
  }

  var IncludeFragmentElement = exports.IncludeFragmentElement = function (_CustomElement2) {
    _inherits(IncludeFragmentElement, _CustomElement2);

    function IncludeFragmentElement() {
      _classCallCheck(this, IncludeFragmentElement);

      return _possibleConstructorReturn(this, (IncludeFragmentElement.__proto__ || Object.getPrototypeOf(IncludeFragmentElement)).call(this));
    }

    _createClass(IncludeFragmentElement, [{
      key: 'attributeChangedCallback',
      value: function attributeChangedCallback(attribute) {
        if (attribute === 'src') {
          // Source changed after attached so replace element.
          if (this._attached) {
            handleData(this);
          }
        }
      }
    }, {
      key: 'connectedCallback',
      value: function connectedCallback() {
        this._attached = true;
        if (this.src) {
          handleData(this);
        }
      }
    }, {
      key: 'disconnectedCallback',
      value: function disconnectedCallback() {
        this._attached = false;
      }
    }, {
      key: 'request',
      value: function request() {
        var src = this.src;
        if (!src) {
          throw new Error('missing src');
        }

        return new Request(src, {
          method: 'GET',
          credentials: 'same-origin',
          headers: {
            Accept: 'text/html'
          }
        });
      }
    }, {
      key: 'load',
      value: function load() {
        var _this2 = this;

        return Promise.resolve().then(function () {
          fire('loadstart', _this2);
          return _this2.fetch(_this2.request());
        }).then(function (response) {
          if (response.status !== 200) {
            throw new Error('Failed to load resource: the server responded with a status of ' + response.status);
          }
          var ct = response.headers.get('Content-Type');
          if (!ct || !ct.match(/^text\/html/)) {
            throw new Error('Failed to load resource: expected text/html but was ' + ct);
          }
          return response;
        }).then(function (response) {
          return response.text();
        }).then(function (data) {
          fire('load', _this2);
          fire('loadend', _this2);
          return data;
        }, function (error) {
          fire('error', _this2);
          fire('loadend', _this2);
          throw error;
        });
      }
    }, {
      key: 'fetch',
      value: function (_fetch) {
        function fetch(_x) {
          return _fetch.apply(this, arguments);
        }

        fetch.toString = function () {
          return _fetch.toString();
        };

        return fetch;
      }(function (request) {
        return fetch(request);
      })
    }, {
      key: 'src',
      get: function get() {
        var src = this.getAttribute('src');
        if (src) {
          var link = this.ownerDocument.createElement('a');
          link.href = src;
          return link.href;
        } else {
          return '';
        }
      },
      set: function set(val) {
        if (val) {
          this.setAttribute('src', val);
        } else {
          this.removeAttribute('src');
        }
      }
    }, {
      key: 'data',
      get: function get() {
        return getData(this);
      }
    }], [{
      key: 'observedAttributes',
      get: function get() {
        return ['src'];
      }
    }]);

    return IncludeFragmentElement;
  }(_CustomElement);

  if (!window.customElements.get('include-fragment')) {
    window.IncludeFragmentElement = IncludeFragmentElement;
    window.customElements.define('include-fragment', IncludeFragmentElement);
  }
});
